import cv2
import tensorflow as tf
from tensorflow import keras
import numpy as np

img_rows = 128
img_cols = 128
num_channels = 3
# ニューラルネットワークモデルの定義

model = keras.models.load_model("detection_weight.h5", compile=False)
model.compile(optimizer="adam",
              loss='categorical_crossentropy',
              metrics=['accuracy'])

### ニューラルネットワークのモデル定義終わり

mirror = True
size = None

# 引数はカメラ番号
cap = cv2.VideoCapture(0)

ret1, frame1 = cap.read()
height, width = frame1.shape[:2] #実験環境ではheight:480, width:640
# 画像サイズは縦，横の順
# size=(640,480) # サイズ指定する場合はここを解除

size_window=(200,200) #ROIのサイズ
stride_row=3
stride_col=3

print(frame1.shape[:2]) # 出力を見るとわかるが，arrayとしては縦，横の順になっている
while True:
    ret, frame = cap.read()
    # mirrorがTrueなら左右反転
    if mirror is True:
        frame = frame[:,::-1]

    # sizeがNoneじゃなく、sizeの次元が2ならsizeが画像サイズとみなしてリサイズ
    if size is not None and len(size) == 2:
        frame = cv2.resize(frame, size)

    # frameにカメラ映像の各フレームの画像データが入っている
    # ここから各画像への処理を書く。
    frame = np.array(frame)  # ndarrayでは四角は描けないのでarrayに変換。
    # pt1を起点としてsize_windowの範囲をROIとする
    # ただし，この座標としては横，縦の順になっていることに注意
    pt1=(80+40,160+20)
    pt2=(pt1[0]+size_window[0],pt1[1]+size_window[1])

    # roi領域を画像として保存し予測の入力とする
    # 配列としての形は縦，横の順で指定する
    roi=cv2.resize(frame[pt1[1]:pt2[1],pt1[0]:pt2[0]],(img_rows,img_cols))
    imgs=[]
    # 分類に使う画像は128×128なのでスケーリングする
    imgs.append(cv2.resize(roi,(img_rows,img_cols)))
    imgs=np.array(imgs)
    imgs=imgs.reshape(imgs.shape[0],img_rows,img_cols,num_channels)
    # 予測して、出力結果をラベルに変換
    predictions = model.predict(imgs)
    result=np.argmax(predictions[0])
    # 確認用出力
    #print(result)

    # ターゲット（予測ラベル0）なら赤、それ以外なら緑でroiの枠を描画
    if result==0:
        cv2.rectangle(frame, pt1, pt2, (0, 0, 255), 3)
    else:
        cv2.rectangle(frame, pt1, pt2, (0, 255, 0), 3)


    # 各画像への処理終わり。

    cv2.imshow('camera cap', frame)
    cv2.imshow('ROI', imgs[0])

    k = cv2.waitKey(5)
    if k == 27: # Escを押したら終了
        break

# 後処理
cap.release()
cv2.destroyAllWindows()